#ifndef __IDE_NET_DYN_H__
#define __IDE_NET_DYN_H__

//----------------------------------------------------------------------------
typedef struct {
    char* name;         // player name
    char* namecolor;    // player name colors (ST only)
    int fragcount;      // player frags
    int ping;           // player ping
    int level;          // player level (ZD only, imho useless)
    int minutesingame;  // player time in game (ZD only)
    bool isbot;         // is player bot (ZD only)
    bool isspectator;   // is player spectator (ZD and ST)
    int team;           // player team number (0-Red, 1-Blue,..) (ZD and ST)
    char* countrycode;  // player country code, 2 symbols: RU,US,... (ZD only)
} PlayerData;

typedef struct {
    int gmflag;         // 0-CS, 1-ZD, 2-ST
    int result;         // connection status (SVMSG_*, in ide_const.h)
    int ping;           // server ping
    char address[MAXADR+1];// server address:port
    char* hostname;     // server name
    char* wadurl;       // PWADs url (ZD and ST)
    char* email;        // server email (ZD and ST)
    char* mapname;      // map name
    int wadnumb;        // number of additional pwads
    char* wadname[MAX_WAD_NUMB]; // names of pwads
    bool wadoptional[MAX_WAD_NUMB]; // true if pwad is optional (ZD only)
    char* wadnamealt[MAX_WAD_NUMB]; // alternative PWADs (ZD)
    int numclients;     // number of clients
    int maxclients;     // max. number of clients
    int numplayers;     // number of players w/o spectators(in CS == numclients)
    int maxplayers;     // max. number of players can join (in CS == maxclients)
    int numbots;        // number of bots (ZD only)
    int botskill;       // bot skill (ST)
    int numspectators;  // number of spectators (ZD and ST)
    int gameskill;      // skill (1-5)
    int gametype;       // game type (0-10) (see below)
    bool gt_instagib;   // game type modifier - instagib (railgun only) - ST
    bool gt_buckshot;   // game type modifier - buckshot (SSG only) - ST
    char* gamename;     // game name: Doom, Doom II, etc..
    char* gameiwad;     // game IWAD
    char* gameiwadalt;  // alternative IWAD (ZD)
    long dmflags;       // dmflags
    long dmflags2;      // dmflags2 (ZD and ST)
    long compatflags;   // compatflags (ST only)
    bool forcepassword; // is server password-protected (ZD and ST)
    bool forcejoinpassword; // is server require password to join game (ST)
    int timelimit;      // timelimit (ZD and ST)
    int timeleft;       // time left to level end (ZD and ST)
    int fraglimit;      // fraglimit (ZD and ST)
    int fragleft;       // frags left to level end (ZD and ST)
    int pointlimit;     // pointlimit (ST teamgames only, 8-10)
    int duellimit;      // duellimit (ST, duels)
    int winlimit;       // winlimit (ST, LMS mode)
    int minplayers;     // add temporary bots (max = minplayers-1) (ZD only)
    bool removebotswhenhumans; // remove all bots when 2nd player join(ZD only)
    int teamscorelimit; // max. teamscore for teamgames (ZD and ST)
    int teamscoreleft;  // teamscore left to end of level (ZD and ST)
    int maxteams;       // number of teams allowed (ZD and ST)
    int teamscore[teamNUMB]; // current teamscores (ZD and ST)
    int teamcolor[teamNUMB]; // custom team colors (ZD)
    char* teamname[teamNUMB];// custom team names (ZD)
    double teamdamage;  // damage to teammates (0-1) (ZD and ST)
    int version;        // server program version - integer (ZD)
    char* sversion;     // server program version - string (ST and ZD)
    double gravity;     // gravity (0 - 800) (ZD only)
    double aircontrol;  // air control (0 - 1) (ZD only)
    bool odafeatures[odfNUMB]; // Odamex setup flags
    int startuptime;    // server startup time (ZD)
    bool acl;           // is server invite-only (ZD)
    int acluser;        // is user invited to play on this server (ZD)
    int voting;         // does server enable voting (ZD)

    PlayerData player[MAX_PLAYER_NUMB];

// internal
    char buf[MAX_UDP_PACKET];
} ServerData;

typedef struct {
    char name[ZD_MAX_NAME+1];       // player name
    char extrainfo[ZD_MAX_EXTRA+1]; // player extrainfo
    int experience;                 // player experience
    int rank;                       // player rank
    int highestrank;                // player highest rank
    double level;                   // player level
} ZDPlayerData;
//----------------------------------------------------------------------------

typedef void (*ShowStepFcn)();

//----------------------------------------------------------------------------
typedef int  (__stdcall *IDE_GetVersion_PTR)(void);
typedef bool (__stdcall *IDE_Init_PTR)(void);
typedef void (__stdcall *IDE_Close_PTR)(void);
typedef int  (__stdcall *IDE_GetLocalPort_PTR)(void);
typedef void (__stdcall *IDE_SetLocalPort_PTR)(int port);
typedef void (__stdcall *IDE_GetMasterAddress_PTR)(int gmflag,char* s);
typedef void (__stdcall *IDE_SetMasterAddress_PTR)(int gmflag,char* s);
typedef bool (__stdcall *IDE_GetWarningMessage_PTR)(char* msg);
typedef void (__stdcall *IDE_SetShowStep_PTR)(ShowStepFcn fn);
typedef void (__stdcall *IDE_MustTerminated_PTR)(bool m);
typedef int  (__stdcall *IDE_GetMasterList_PTR)(int gmflag);
typedef bool (__stdcall *IDE_GetMasterListServer_PTR)(int i,char* sv);
typedef bool (__stdcall *IDE_GetServerInfo_PTR)(char* address,int gmflag,ServerData* svi);
typedef bool (__stdcall *IDE_GetServerGroup_PTR)(ServerData *svi[],int svnumb,int senddelay,int recvdelay);

extern IDE_GetVersion_PTR             IDE_GetVersion;
extern IDE_Init_PTR                   IDE_Init;
extern IDE_Close_PTR                  IDE_Close;
extern IDE_GetLocalPort_PTR           IDE_GetLocalPort;
extern IDE_SetLocalPort_PTR           IDE_SetLocalPort;
extern IDE_GetMasterAddress_PTR       IDE_GetMasterAddress;
extern IDE_SetMasterAddress_PTR       IDE_SetMasterAddress;
extern IDE_GetWarningMessage_PTR      IDE_GetWarningMessage;
extern IDE_SetShowStep_PTR            IDE_SetShowStep;
extern IDE_MustTerminated_PTR         IDE_MustTerminated;
extern IDE_GetMasterList_PTR          IDE_GetMasterList;
extern IDE_GetMasterListServer_PTR    IDE_GetMasterListServer;
extern IDE_GetServerInfo_PTR          IDE_GetServerInfo;
extern IDE_GetServerGroup_PTR         IDE_GetServerGroup;
//----------------------------------------------------------------------------
bool Load_IDE_Net_DLL(void);
void Free_IDE_Net_DLL(void);

#endif
